#!/usr/bin/env bash
# Description: ラリホー: lrを正しくスペルアウトできるかゲーム

:<<"SCREENLAYOUT"
(正解/全問)
日本語問題文
カタカナ読み
---
順にlr以外の文字が表示される 0.5secごと。lr箇所で停止、プロンプト、[lr]入力 ... あっても間違っても次の文字
単語が全部表示されたら、正解表示
SCREENLAYOUT

# 問題と、回答はすべて記録して、最後に並べて表示

SCOREFILE="rallyho.scr"

## VT100 ANSI Escape Sequences
CSI='\033['             # エスケープシーケンス開始
CUB="${CSI}1D"          # 1文字カーソルを戻す
CGREEN="${CSI}0;32m"    # 文字を緑
CYELLOW="${CSI}0;33m"   # 文字を黄
CBGGREEN="${CSI}0;42m"  # 背景を緑
CBGRED="${CSI}0;41m"    # 背景を赤
C0="${CSI}0m"           # 表示文字属性色をリセット
CENC="${CSI}?25h"       # カーソル表示
CNOC="${CSI}?25l"       # カーソル非表示
CKILL="${CSI}K"         # 行を消去

questions=()

read_q() { IFS=$'\n' questions=( $(cat "$@") ); }
get_q() { awk -F $'\t' '{if (NF>2) printf("%s\n", $3); printf("%s\n", $2);}'<<<"$1"; }
get_ca() { awk -F $'\t' '{print $1}'<<<"$1"; }

init_tty() { stty -icanon -echo min 1 -ixon ; echo -e "$CNOC"; }
flush_tty() { stty -icanon min 0 ; while read ; do read ; done ; stty min 1 ; }
finit_tty() { echo -e "$C0$CENC"; stty icanon echo eof '^d' ixon ; }

getlr() { # グローバル変数aに、lrを入れてreturn
    local s=5
    flush_tty
    echo -en "${CNOC}${CBGRED}${s}${CUB}"
    while [ $s -gt 0 ]; do
        let s--
        while : ; do
            IFS= read -s -n 1 -t 1 a
            case "$a" in
                [LRlr])
                    echo -en "${CGREEN}${a}${C0}${CENC}"
                    return 0
                    ;;
                "Q") # read -t中は割込キー無効
                    return -1
                    ;;
                *)
                    echo -en "${CBGRED}${s}${C0}${CUB}"
                    break
                    ;;
            esac
        done
    done
    a="_" # timeoutの場合は "_" をaに入れてreturn
    echo -en "${CBGRED}${a}${C0}${CENC}"
}

wait_til_key() {
    local s=${1-5}
    local p=" > "
    local a
    local mesg0="Start in "
    local mesg1=" :カウントダウン中にスペースキーを入力すれば開始。'Q'で中止。"
    local mesg2=":カウントダウン中に'l', 'r'を入力して解答。'Q'で中断"
    echo -en "${CNOC}"
    while [ $s -gt 0 ] ; do
        printf "%b%s%2d sec.%b%s%b\r" "$C0" "$mesg0" $s "$CYELLOW" "$mesg1" "$C0"
        IFS= read -s -n 1 -t 1
        case "$REPLY" in
            ' ' ) break ;;
            Q) # read -t中は割込無効
                return -1 ;;
        esac
        let s--
    done
    printf "%b%b%b%s%b\n" "$CKILL" "$CGREEN" "$CENC" "$mesg2" "$C0"
}

trap_exit() {
    finit_tty
    echo "終了します: "
    exit
}

usage_exit() {
    printf "%s [-n #] <dictionaly>\n" $0
    echo "   -n # : 問題数 (default=$N)"
    exit
}

N=10 # 出題数
while getopts :n: OPT ; do
    case $OPT in
        n) N=$(expr ${OPTARG} + 0 2> /dev/null) || N=2 ;;
        *) usage_exit ;;
    esac
done
shift $((OPTIND - 1))

dicts=()
while [ -n "$1" ]; do
    [ -f "$1" ] && dicts+=("$1")
    shift
done
if [ ${#dicts[@]} -gt 0 ]; then
    read_q "${dicts[@]}"
else
    read_q "lrdict.txt" # default dictionary
fi

trap '' INT TERM QUIT
init_tty

REGEX_comment="^#"
REGEX_lr='[LRlr]'

c=0 # 正解数
declare -a r cas anss
for i in $(seq $N) ; do
    
    while : ; do
        n=$(( RANDOM % ${#questions[@]} ))  # 問題番号
        [[ "${questions[$n]}" =~ $REGEX_comment ]] && continue
        ca=$(get_ca "${questions[$n]}")     # 正解
        break
    done

    printf "\n%d/%d\n" $i $N
    get_q "${questions[$n]}"
    echo '-- -- -- -- --'

    p=0
    ans=""

    wait_til_key 10
    [ $? -ne 0 ] && trap_exit

    while [ $p -lt ${#ca} ]; do
        n="${ca:$p:1}"
        if [[ $n =~ $REGEX_lr ]]; then
            getlr
            [ $? -ne 0 ] && trap_exit
            ans="$ans$a"
        else
            sleep 0.3 ; echo -n "$n"
            ans="$ans$n"
        fi
        let p++
    done
    echo
    if [ "${ca,,}" = "${ans,,}" ]; then
        echo -e "${CBGGREEN}${ca,,}${C0}"
        r+=("✔")
        let c++
    else
        echo -e "${CBGRED}${ca,,}${C0}"
        r+=("✘")
    fi
    cas+=(${ca,,})
    anss+=(${ans,,})
    sleep 1
done

printf "%s: %d/%d\n" $(date +"%Y/%m/%d %H:%M:%S") >> "$SCOREFILE"

sleep 0.2 ; echo ; sleep 0.2 ; echo -n '終了 '
for i in {1..10} ; do sleep 0.1 ; echo -n '.' ; done
sleep 0.2 ; printf "\n%d問中 %d問 正解\n" $N $c ; sleep 1

echo "  # 正誤 |             解答               |             正解               |"
for i in $(seq 0 $(( N - 1 )) ) ; do
    printf "%3d %4s   |%-32s|%-32s|\n" $((i+1)) "${r[$i]}" "${anss[$i]}" "${cas[$i]}"
done

finit_tty
